package edu.uoc.baserrikoa.Control.carrito;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.baserrikoa.Control.MenuManagementActivity;
import edu.uoc.baserrikoa.R;
import edu.uoc.baserrikoa.model.CarritoContent;

import static edu.uoc.baserrikoa.R.drawable.rounded_rectangle;

/**
 * Esta actividad nos permite ver todos los elementos que el usuario ha seleccionado
 * para comprar. Dentro de la actividad podremos cambiar las cantidades y validar
 * la compra.
 */
public class CarritoActivity extends MenuManagementActivity {

    private final static String TAG = "Carrito";
    String compraId;
    private SimpleItemRecyclerViewAdapter adapter;
    private TextView textTotal;
    Button comprar;

    @SuppressLint("ResourceAsColor")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_carrito_list);

        Toolbar toolbar = (Toolbar) findViewById(R.id.my_toolbar);
        setSupportActionBar(toolbar);
        toolbar.setTitle(getTitle());


        RecyclerView recyclerView = (RecyclerView) findViewById(R.id.carrito_list);
        RecyclerView.LayoutManager layoutManager = new LinearLayoutManager(CarritoActivity.this);
        recyclerView.setLayoutManager(layoutManager);
        setupRecyclerView(recyclerView);

        textTotal = (TextView) findViewById(R.id.carrito_total);



        comprar = (Button) findViewById(R.id.carrito_comprar);
        comprar.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent(CarritoActivity.this, PaymentActivity.class);
                startActivity(intent);
            }
        });

        activarBoton();

        adapter.setItems(CarritoContent.getCarrito());
        actualicarTotal();
    }


    public void onResume(){
        super.onResume();

        activarBoton();
    }


    private void activarBoton(){
        Float coste = 0f;
        for (CarritoContent.CarritoItem item:CarritoContent.getCarrito()) {
            coste += Float.parseFloat(item.cantidad) *  Float.parseFloat(item.coste);
        }
        if(coste > 0f) {
            comprar.setEnabled(true);
            comprar.setBackground(getResources().getDrawable(rounded_rectangle));
        }else {
            comprar.setBackgroundColor(Color.GRAY);
            comprar.setEnabled(false);
        }
    }

    public void actualicarTotal(){
        Float coste = 0f;
        for (CarritoContent.CarritoItem item:CarritoContent.getCarrito()) {
            coste += Float.parseFloat(item.cantidad) *  Float.parseFloat(item.coste);
        }
        textTotal.setText(String.format("%.2f", coste));
        activarBoton();
    }


    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
        adapter = new SimpleItemRecyclerViewAdapter(new ArrayList<CarritoContent.CarritoItem>());
        recyclerView.setAdapter(adapter);
    }

    public class SimpleItemRecyclerViewAdapter
            extends RecyclerView.Adapter<SimpleItemRecyclerViewAdapter.ViewHolder> {

        private List<CarritoContent.CarritoItem> mValues;


        public SimpleItemRecyclerViewAdapter(List<CarritoContent.CarritoItem> items) {
            mValues = items;
        }

        public void setItems(List<CarritoContent.CarritoItem> items) {
            mValues = items;
            notifyDataSetChanged();
        }


        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = null;

            view = LayoutInflater.from(parent.getContext())
                        .inflate(R.layout.carrito_list_content, parent, false);

            return new ViewHolder(view);
        }

        /**
         * Clase para gestionar todos los elementos del carrito
         */
        @Override
        public void onBindViewHolder(final ViewHolder holder, final int position) {
            holder.mItem = mValues.get(position);
            holder.mCantidadView.setText(holder.mItem.cantidad);
            holder.mNombreView.setText(holder.mItem.nombre + " ("+ holder.mItem.coste +" €)");
            recalcularCoste(holder, holder.mItem);

            holder.bMas.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    Integer cantidad = Integer.parseInt(holder.mItem.cantidad);
                    cantidad++;
                    holder.mCantidadView.setText(Integer.toString(cantidad));
                    holder.mItem.cantidad = cantidad.toString();
                    holder.mItem.save();
                    recalcularCoste(holder, holder.mItem);
                    actualicarTotal();
                }
            });
            holder.bMenos.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    Integer cantidad = Integer.parseInt(holder.mItem.cantidad);
                    if(!cantidad.equals(0)) {
                        cantidad--;
                        holder.mCantidadView.setText(Integer.toString(cantidad));
                        holder.mItem.cantidad = cantidad.toString();
                        holder.mItem.save();
                        recalcularCoste(holder, holder.mItem);
                        actualicarTotal();
                    }
                }
            });

            holder.bBorrar.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    holder.mItem.delete();
                    setItems(CarritoContent.getCarrito());
                    actualicarTotal();
                }
            });

        }

        private void recalcularCoste(ViewHolder holder, CarritoContent.CarritoItem item){
            Float coste = Float.parseFloat(item.cantidad) *  Float.parseFloat(item.coste);
            holder.mCosteView.setText(String.format("%.2f", coste));
        }



        @Override
        public int getItemCount() {
            return mValues.size();
        }

        /**
         * Clase para gestionar cada uno de los elementos del carrito
         */
        public class ViewHolder extends RecyclerView.ViewHolder {
            public final View mView;
            public final TextView mNombreView;
            public final TextView mCosteView;
            public final TextView mCantidadView;
            public final ImageButton bMas;
            public final ImageButton bMenos;
            public final ImageButton bBorrar;

            public CarritoContent.CarritoItem mItem;

            public ViewHolder(View view) {
                super(view);
                mView = view;
                mNombreView = (TextView) view.findViewById(R.id.carrito_nombre);
                mCosteView = (TextView) view.findViewById(R.id.carrito_coste);
                mCantidadView = (TextView) view.findViewById(R.id.carrito_cantidad);
                bMas = (ImageButton) view.findViewById(R.id.carro_mas);
                bMenos = (ImageButton) view.findViewById(R.id.carro_menos);
                bBorrar = (ImageButton) view.findViewById(R.id.carro_borrar);
            }
        }
    }


}
