package edu.uoc.baserrikoa.Control.comprasAnteriores;

import android.app.ActivityOptions;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.uoc.baserrikoa.Control.MenuManagementActivity;
import edu.uoc.baserrikoa.R;
import edu.uoc.baserrikoa.model.CompraContent;
import edu.uoc.baserrikoa.model.SesionData;

/**
 * Clase que gestiona las compras realizadas por el usuario en conexiones anteriores, permitiendo
 * ver la fecha de la compra y el importe gastado.
 */
public class CompraListActivity extends MenuManagementActivity {

    private final static String TAG = "CompraListActivity";

    /**
     * Whether or not the activity is in two-pane mode, i.e. running on a tablet
     * device.
     */

    private SimpleItemRecyclerViewAdapter adapter;
    private DatabaseReference myRef;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_compra_list);

        Toolbar toolbar = (Toolbar) findViewById(R.id.my_toolbar);
        setSupportActionBar(toolbar);
        toolbar.setTitle(getTitle());


        RecyclerView recyclerView = (RecyclerView) findViewById(R.id.compra_list);
        RecyclerView.LayoutManager layoutManager = new LinearLayoutManager(CompraListActivity.this);
        recyclerView.setLayoutManager(layoutManager);
        setupRecyclerView(recyclerView);


        if(SesionData.firebaseConected) downloadCompras();


    }

    /*
    * Función para descargar desde Firebase todas las compras realizadas por el usuario con anterioridad.*/
    private void downloadCompras() {

        myRef = SesionData.database.getReference("compras").child(SesionData.userID);//Coger solo la información de nuestro usuario
        // Read from the database
        myRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                getComprasFromDataSnapshot(dataSnapshot);

            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.w(TAG, "Failed to read value.", error.toException());
                getComprasFromDB();
            }
        });
    }

    /**
     * Función para cargar desde nuestra base de datos, en caso de haber problemas de conexión,
     * las compras realizadas con anterioridad.
     */
    private void getComprasFromDB() {
        List<CompraContent.CompraItem> values = CompraContent.getCompras();
        adapter.setItems(values);
    }

    /*
    * Función para recibir desde Firebase todas las compras realizadas con anterioridad
    * */
    private void getComprasFromDataSnapshot(DataSnapshot dataSnapshot) {
        GenericTypeIndicator<ArrayList<CompraContent.CompraItem>> genericTypeIndicator =
                new GenericTypeIndicator<ArrayList<CompraContent.CompraItem>>() {};

        Map<String, CompraContent.CompraItem> td = new HashMap<String, CompraContent.CompraItem>();
        for (DataSnapshot jobSnapshot: dataSnapshot.getChildren()) {
            CompraContent.CompraItem job = jobSnapshot.getValue(CompraContent.CompraItem.class);
            td.put(jobSnapshot.getKey(), job);
        }

        ArrayList<CompraContent.CompraItem> values = new ArrayList<>(td.values());
        // Save data in database
        if(values != null) {

            for (CompraContent.CompraItem pItem : values) {
                if (!CompraContent.exists(pItem)) {
                    pItem.save();
                }
            }

            adapter.setItems(values);
        }
    }

    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
        adapter = new SimpleItemRecyclerViewAdapter(new ArrayList<CompraContent.CompraItem>());
        recyclerView.setAdapter(adapter);
    }

    public class SimpleItemRecyclerViewAdapter
            extends RecyclerView.Adapter<SimpleItemRecyclerViewAdapter.ViewHolder> {

        private List<CompraContent.CompraItem> mValues;


        public SimpleItemRecyclerViewAdapter(List<CompraContent.CompraItem> items) {
            mValues = items;
        }

        public void setItems(List<CompraContent.CompraItem> items) {
            mValues = items;
            notifyDataSetChanged();
        }


        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = null;

            view = LayoutInflater.from(parent.getContext())
                        .inflate(R.layout.compra_list_content, parent, false);

            return new ViewHolder(view);
        }

        /*
        * Clase para gestionar todas las compras realizadas con anterioridad
        * */
        @Override
        public void onBindViewHolder(final ViewHolder holder, final int position) {
            holder.mItem = mValues.get(position);
            holder.mCosteView.setText(mValues.get(position).coste);
            holder.mFechaView.setText(mValues.get(position).fecha);


            holder.mView.setTag(position);
            holder.mView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    int currentPos = (int) v.getTag();
                    CompraContent.CompraItem compra = CompraContent.getCompras().get(currentPos);
                    Log.d("LIST: ",""+currentPos);
                    Bundle animation = ActivityOptions.makeCustomAnimation(CompraListActivity.this, R.anim.translate_in_bottom,
                            R.anim.translate_out_bottom).toBundle();
                    Intent intent = new Intent(v.getContext(), ProductosCompraListActivity.class);
                    intent.putExtra("compra_id", compra.idcompra);
                    intent.putExtra("fecha", compra.fecha);
                    intent.putExtra("total", compra.coste);
                    v.getContext().startActivity(intent, animation);

                }
            });
        }

        private void chooseCompra(int currentPos, Context context){

        }



        @Override
        public int getItemCount() {
            return mValues.size();
        }

        public class ViewHolder extends RecyclerView.ViewHolder {
            public final View mView;
            public final TextView mFechaView;
            public final TextView mCosteView;
            public CompraContent.CompraItem mItem;

            public ViewHolder(View view) {
                super(view);
                mView = view;
                mFechaView = (TextView) view.findViewById(R.id.compra_fecha);
                mCosteView = (TextView) view.findViewById(R.id.compra_coste);
            }
        }
    }

    public void onResume(){
        super.onResume();
        RecyclerView recyclerView = (RecyclerView) findViewById(R.id.compra_list);
        RecyclerView.LayoutManager layoutManager = new LinearLayoutManager(CompraListActivity.this);
        recyclerView.setLayoutManager(layoutManager);
        setupRecyclerView(recyclerView);


        if(SesionData.firebaseConected) downloadCompras();
    }

}
