package edu.uoc.baserrikoa.Control.comprasAnteriores;

import android.content.Context;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.uoc.baserrikoa.Control.MenuManagementActivity;
import edu.uoc.baserrikoa.R;
import edu.uoc.baserrikoa.model.ProductosCompraContent;
import edu.uoc.baserrikoa.model.SesionData;

/**
 * Clase que gestiona los productos que se compraron en una compra anterior.
 */
public class ProductosCompraListActivity extends MenuManagementActivity {

    private final static String TAG = "ProductosC";
    String compraId;
    private SimpleItemRecyclerViewAdapter adapter;
    private DatabaseReference myRef;
    public Toolbar toolbar;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_productos_compra_list);

        compraId = getIntent().getStringExtra("compra_id");
        String fecha = getIntent().getStringExtra("fecha");
        String total = getIntent().getStringExtra("total");
        toolbar = (Toolbar) findViewById(R.id.my_toolbar);
        setSupportActionBar(toolbar);

        TextView tFecha = (TextView) findViewById(R.id.product_compra_fecha);
        tFecha.setText(fecha);
        TextView tTotal = (TextView) findViewById(R.id.product_compra_total);
        tTotal.setText(total);

        RecyclerView recyclerView = (RecyclerView) findViewById(R.id.productosCompra_list);
        RecyclerView.LayoutManager layoutManager = new LinearLayoutManager(ProductosCompraListActivity.this);
        recyclerView.setLayoutManager(layoutManager);
        setupRecyclerView(recyclerView);

        if(SesionData.firebaseConected){
            downloadProductosCompras();
        }


    }

    /*Descargar los productos comprados en determinada compra anterior.*/
    private void downloadProductosCompras() {

        myRef = SesionData.database.getReference("productosCompras").child(compraId);//Coger solo la información de nuestro usuario
        // Read from the database
        myRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                getProductosComprasFromDataSnapshot(dataSnapshot);

            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.w(TAG, "Failed to read value.", error.toException());
                getProductosComprasFromDB();
            }
        });
    }

    /*Cargar de la base de datos los productos de una compra anterior en caso de que haya problemas
    con la conexión.*/
    private void getProductosComprasFromDB() {
        List<ProductosCompraContent.ProductosCompraItem> values = ProductosCompraContent.getProductosCompras();
        adapter.setItems(values);
    }

    /*Gestionar los datos enviados por Firebase de la compra seleccionada*/
    private void getProductosComprasFromDataSnapshot(DataSnapshot dataSnapshot) {

        GenericTypeIndicator<ArrayList<ProductosCompraContent.ProductosCompraItem>> genericTypeIndicator =
                new GenericTypeIndicator<ArrayList<ProductosCompraContent.ProductosCompraItem>>() {};
        Map<String, ProductosCompraContent.ProductosCompraItem> td = new HashMap<String, ProductosCompraContent.ProductosCompraItem>();
        for (DataSnapshot jobSnapshot: dataSnapshot.getChildren()) {
            ProductosCompraContent.ProductosCompraItem comprados = jobSnapshot.getValue(ProductosCompraContent.ProductosCompraItem.class);
            td.put(jobSnapshot.getKey(), comprados);
        }

        ArrayList<ProductosCompraContent.ProductosCompraItem> values = new ArrayList<>(td.values());;
        // Save data in database
        if(values != null) {
            for (ProductosCompraContent.ProductosCompraItem pItem : values) {
                if (!ProductosCompraContent.exists(pItem)) {
                    pItem.save();
                }
            }

            adapter.setItems(values);
        }
    }

    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
        adapter = new SimpleItemRecyclerViewAdapter(new ArrayList<ProductosCompraContent.ProductosCompraItem>());
        recyclerView.setAdapter(adapter);
    }

    public class SimpleItemRecyclerViewAdapter
            extends RecyclerView.Adapter<SimpleItemRecyclerViewAdapter.ViewHolder> {

        private List<ProductosCompraContent.ProductosCompraItem> mValues;


        public SimpleItemRecyclerViewAdapter(List<ProductosCompraContent.ProductosCompraItem> items) {
            mValues = items;
        }

        public void setItems(List<ProductosCompraContent.ProductosCompraItem> items) {
            mValues = items;
            notifyDataSetChanged();
        }


        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = null;

            view = LayoutInflater.from(parent.getContext())
                        .inflate(R.layout.productos_compra_list_content, parent, false);

            return new ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final ViewHolder holder, final int position) {
            holder.mItem = mValues.get(position);

            holder.mCosteView.setText(mValues.get(position).coste);
            holder.mNombreView.setText(mValues.get(position).nombre+" ( x"+mValues.get(position).cantidad+" )");
        }

        private void chooseProductosCompra(int currentPos, Context context){

        }



        @Override
        public int getItemCount() {
            return mValues.size();
        }

        /**
         * Clase para gestionar cada uno de los elementos de la compra
         */
        public class ViewHolder extends RecyclerView.ViewHolder {
            public final View mView;
            public final TextView mNombreView;
            public final TextView mCosteView;
            public ProductosCompraContent.ProductosCompraItem mItem;

            public ViewHolder(View view) {
                super(view);
                mView = view;
                mNombreView = (TextView) view.findViewById(R.id.productosCompra_nombre);
                mCosteView = (TextView) view.findViewById(R.id.productosCompra_coste);
            }
        }
    }

}
