package edu.uoc.baserrikoa.Control.ofertas;

import android.app.ActivityOptions;
import android.app.NotificationManager;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.Snackbar;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.baserrikoa.Control.MenuManagementActivity;
import edu.uoc.baserrikoa.R;
import edu.uoc.baserrikoa.model.DescuentoContent;
import edu.uoc.baserrikoa.model.SesionData;

/**
 * Esta actividad representa una lista de descuentos
 */
public class DescuentoListActivity extends MenuManagementActivity {

    private final static String TAG = "DescuentoListActivity";

    private SimpleItemRecyclerViewAdapter adapter;
    private DatabaseReference myRef;
    private SwipeRefreshLayout swipeContainer;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_descuento_list);

        Toolbar toolbar = (Toolbar) findViewById(R.id.my_toolbar);
        setSupportActionBar(toolbar);
        toolbar.setTitle(getTitle());

        swipeContainer = (SwipeRefreshLayout) findViewById(R.id.swipeContainer);
        // Setup refresh listener which triggers new data loading
        swipeContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                refreshDescuentoList();
            }
        });
        // Configure the refreshing colors
        swipeContainer.setColorSchemeResources(android.R.color.holo_blue_bright,
                android.R.color.holo_green_light,
                android.R.color.holo_orange_light,
                android.R.color.holo_red_light);

        RecyclerView recyclerView = (RecyclerView) findViewById(R.id.descuento_list);
        RecyclerView.LayoutManager layoutManager = new GridLayoutManager(DescuentoListActivity.this, 2);
        recyclerView.setLayoutManager(layoutManager);
        setupRecyclerView(recyclerView);

        onPendingIntent();
        if(SesionData.firebaseConected) downloadDescuentos();
    }

    protected void onPendingIntent() {
        if (getIntent() != null && getIntent().getAction() != null) {
            int descuentoID = getIntent().getIntExtra("item_id", 0);
            if (getIntent().getAction().equalsIgnoreCase(Intent.ACTION_SEND)) {
                // Ver información del descuento
                Bundle animation = ActivityOptions.makeCustomAnimation(DescuentoListActivity.this, R.anim.translate_in_bottom,
                            R.anim.translate_out_bottom).toBundle();
                Intent intent = new Intent(this, DescuentoDetailActivity.class);
                intent.putExtra(DescuentoDetailFragment.ARG_ITEM_ID, descuentoID);
                startActivity(intent, animation);
            }
            NotificationManager mNotificationManager =
                    (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
            // Eliminar la notificación
            mNotificationManager.cancel(3);
        }
    }

    /*Descargar lista de descuentos del usuario desde el servidor*/
    private void downloadDescuentos() {
        swipeContainer.setRefreshing(true);

        myRef = SesionData.database.getReference("descuentos").child(SesionData.userID);
        // Read from the database
        myRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                getDescuentosFromDataSnapshot(dataSnapshot);

            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.w(TAG, "Failed to read value.", error.toException());
                getDescuentosFromDB();
            }
        });
    }

    /*Refrescar lista de descuentos del usuario desde el servidor*/
    private void refreshDescuentoList() {
        if (myRef != null) {
            myRef.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    getDescuentosFromDataSnapshot(dataSnapshot);
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {
                    // Failed to read value
                    Log.w(TAG, "Failed to read value.", databaseError.toException());
                    getDescuentosFromDB();
                }
            });
        } else {
            Log.w(TAG, "My Ref null");
            getDescuentosFromDB();
        }
    }

    private void getDescuentosFromDB() {
        List<DescuentoContent.DescuentoItem> values = DescuentoContent.getDescuentos();
        adapter.setItems(values);
        swipeContainer.setRefreshing(false);
        Snackbar.make(swipeContainer, "Descuentos updated from DB", Snackbar.LENGTH_LONG).show();
    }

    private void getDescuentosFromDataSnapshot(DataSnapshot dataSnapshot) {
        // This method is called once with the initial value and again
        // whenever data at this location is updated.
        GenericTypeIndicator<ArrayList<DescuentoContent.DescuentoItem>> genericTypeIndicator =
                new GenericTypeIndicator<ArrayList<DescuentoContent.DescuentoItem>>() {};
        ArrayList<DescuentoContent.DescuentoItem> values = dataSnapshot.getValue(genericTypeIndicator);
        // Save data in database
        if(values != null) {
            for (DescuentoContent.DescuentoItem pItem : values) {
                if (!DescuentoContent.exists(pItem)) {
                    pItem.save();
                }
            }

        adapter.setItems(values);
        swipeContainer.setRefreshing(false);
        Snackbar.make(swipeContainer, "Descuentos updated from Firebase", Snackbar.LENGTH_LONG).show();
        }else{
            swipeContainer.setRefreshing(false);
            Snackbar.make(swipeContainer, "No se han encontrado ofertas", Snackbar.LENGTH_LONG).show();
        }
    }

    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
        adapter = new SimpleItemRecyclerViewAdapter(new ArrayList<DescuentoContent.DescuentoItem>());
        recyclerView.setAdapter(adapter);
    }

    public class SimpleItemRecyclerViewAdapter
            extends RecyclerView.Adapter<SimpleItemRecyclerViewAdapter.ViewHolder> {

        private List<DescuentoContent.DescuentoItem> mValues;


        public SimpleItemRecyclerViewAdapter(List<DescuentoContent.DescuentoItem> items) {
            mValues = items;
        }

        public void setItems(List<DescuentoContent.DescuentoItem> items) {
            mValues = items;
            notifyDataSetChanged();
        }


        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = null;

            view = LayoutInflater.from(parent.getContext())
                        .inflate(R.layout.descuento_list_content, parent, false);

            return new ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final ViewHolder holder, final int position) {
            holder.mItem = mValues.get(position);
            holder.mNombreView.setText(mValues.get(position).nombre);
            holder.mCantidadView.setText(mValues.get(position).validez);
            if (holder.mImagenView != null) {
                Picasso.with(DescuentoListActivity.this).load(mValues.get(position).getImagen())
                            .into(holder.mImagenView);

            }

            holder.mView.setTag(position);
            holder.mView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    int currentPos = (int) v.getTag();
                    Log.d("LIST: ",""+currentPos);
                    Bundle animation = ActivityOptions.makeCustomAnimation(DescuentoListActivity.this, R.anim.translate_in_bottom,
                                R.anim.translate_out_bottom).toBundle();
                    Intent intent = new Intent(v.getContext(), DescuentoDetailActivity.class);
                    intent.putExtra(DescuentoDetailFragment.ARG_ITEM_ID, currentPos);
                    v.getContext().startActivity(intent, animation);


                }
            });
        }

        private void chooseDescuento(int currentPos, Context context){

        }



        @Override
        public int getItemCount() {
            return mValues.size();
        }

        public class ViewHolder extends RecyclerView.ViewHolder {
            public final View mView;
            public final TextView mNombreView;
            public final TextView mCantidadView;
            public final ImageView mImagenView;
            public DescuentoContent.DescuentoItem mItem;

            public ViewHolder(View view) {
                super(view);
                mView = view;
                mNombreView = (TextView) view.findViewById(R.id.descuento_nombre);
                mCantidadView = (TextView) view.findViewById(R.id.descuento_cantidad);
                mImagenView = (ImageView) view.findViewById(R.id.descuento_image);
            }

            @Override
            public String toString() {
                return super.toString() + " '" + mNombreView.getText() + "'";
            }
        }
    }
}
