package edu.uoc.baserrikoa.Control.productor.Producto;

import android.app.ActivityOptions;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.baserrikoa.Control.carrito.CarritoActivity;
import edu.uoc.baserrikoa.Control.MenuManagementActivity;
import edu.uoc.baserrikoa.R;
import edu.uoc.baserrikoa.model.CarritoContent;
import edu.uoc.baserrikoa.model.ProductContent;
import edu.uoc.baserrikoa.model.SesionData;

/**
 * An activity representing a list of Products. This activity
 * has different presentations for handset and tablet-size devices. On
 * handsets, the activity presents a list of items, which when touched,
 * lead to a {@link ProductDetailActivity} representing
 * item details. On tablets, the activity presents the list of items and
 * item details side-by-side using two vertical panes.
 */
public class ProductListActivity extends MenuManagementActivity {

    private final static String TAG = "ProductListActivity";

    /**
     * Whether or not the activity is in two-pane mode, i.e. running on a tablet
     * device.
     */
    private SimpleItemRecyclerViewAdapter adapter;
    private DatabaseReference myRef;
    private SwipeRefreshLayout swipeContainer;
    String vendedorId;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_product_list);

        vendedorId = getIntent().getStringExtra("vendedor_id");

        Toolbar toolbar = (Toolbar) findViewById(R.id.my_toolbar);
        setSupportActionBar(toolbar);
        toolbar.setTitle(getTitle());

        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab_carrito);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent(ProductListActivity.this, CarritoActivity.class);
                startActivity(intent);
            }
        });

        swipeContainer = (SwipeRefreshLayout) findViewById(R.id.swipeContainer);
        // Setup refresh listener which triggers new data loading
        swipeContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                refreshProductList();
            }
        });
        // Configure the refreshing colors
        swipeContainer.setColorSchemeResources(android.R.color.holo_blue_bright,
                android.R.color.holo_green_light,
                android.R.color.holo_orange_light,
                android.R.color.holo_red_light);

        RecyclerView recyclerView = (RecyclerView) findViewById(R.id.product_list);
        RecyclerView.LayoutManager layoutManager = new GridLayoutManager(ProductListActivity.this, 2);
        recyclerView.setLayoutManager(layoutManager);
        setupRecyclerView(recyclerView);

        if(SesionData.firebaseConected) downloadProducts();


    }

    private void downloadProducts() {
        swipeContainer.setRefreshing(true);

        myRef = SesionData.database.getReference("products").child(vendedorId);
        // Read from the database
        myRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                getProductsFromDataSnapshot(dataSnapshot);

            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.w(TAG, "Failed to read value.", error.toException());
                getProductsFromDB();
            }
        });
    }


    private void refreshProductList() {
        if (myRef != null) {
            myRef.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    getProductsFromDataSnapshot(dataSnapshot);
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {
                    // Failed to read value
                    Log.w(TAG, "Failed to read value.", databaseError.toException());
                    getProductsFromDB();
                }
            });
        } else {
            Log.w(TAG, "My Ref null");
            getProductsFromDB();
        }
    }

    private void getProductsFromDB() {
        List<ProductContent.ProductItem> values = ProductContent.getProducts();
        adapter.setItems(values);
        swipeContainer.setRefreshing(false);
        Snackbar.make(swipeContainer, "Products updated from DB", Snackbar.LENGTH_LONG).show();
    }

    private void getProductsFromDataSnapshot(DataSnapshot dataSnapshot) {
        // This method is called once with the initial value and again
        // whenever data at this location is updated.
        GenericTypeIndicator<ArrayList<ProductContent.ProductItem>> genericTypeIndicator =
                new GenericTypeIndicator<ArrayList<ProductContent.ProductItem>>() {};
        ArrayList<ProductContent.ProductItem> values = dataSnapshot.getValue(genericTypeIndicator);
        // Save data in database

        if(values != null) {
            for (ProductContent.ProductItem pItem : values) {
                if (!ProductContent.exists(pItem)) {
                    pItem.save();
                }
            }

            adapter.setItems(values);
            swipeContainer.setRefreshing(false);
            Snackbar.make(swipeContainer, "Productos cargados", Snackbar.LENGTH_LONG).show();
        }else{
            swipeContainer.setRefreshing(false);
            Snackbar.make(swipeContainer, "No se han podido cargar los productos", Snackbar.LENGTH_LONG).show();
        }
    }

    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
        adapter = new SimpleItemRecyclerViewAdapter(new ArrayList<ProductContent.ProductItem>());
        recyclerView.setAdapter(adapter);
    }

    public class SimpleItemRecyclerViewAdapter
            extends RecyclerView.Adapter<SimpleItemRecyclerViewAdapter.ViewHolder> {

        private List<ProductContent.ProductItem> mValues;


        public SimpleItemRecyclerViewAdapter(List<ProductContent.ProductItem> items) {
            mValues = items;
        }

        public void setItems(List<ProductContent.ProductItem> items) {
            mValues = items;
            notifyDataSetChanged();
        }


        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = null;

            view = LayoutInflater.from(parent.getContext())
                        .inflate(R.layout.product_list_content, parent, false);

            return new ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final ViewHolder holder, final int position) {
            holder.mItem = mValues.get(position);
            holder.mNombreView.setText(mValues.get(position).nombre);
            holder.mPrecioView.setText(mValues.get(position).precio.toString());
            if (holder.mImagenView != null) {
                Picasso.with(ProductListActivity.this).load(mValues.get(position).getImagen())
                            .into(holder.mImagenView);

            }

            holder.mView.setTag(position);
            holder.mView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    int currentPos = (int) v.getTag();
                    Log.d("LIST: ",""+currentPos);
                    Bundle animation = ActivityOptions.makeCustomAnimation(ProductListActivity.this, R.anim.translate_in_bottom,
                                R.anim.translate_out_bottom).toBundle();
                    Intent intent = new Intent(v.getContext(), ProductDetailActivity.class);
                    intent.putExtra(ProductDetailFragment.ARG_ITEM_ID, currentPos);
                    v.getContext().startActivity(intent, animation);
                }
            });

            holder.bMas.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    holder.cantidad++;
                    holder.mCantidad.setText(Integer.toString(holder.cantidad));
                }
            });
            holder.bMenos.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if(!holder.cantidad.equals(0)) {
                        holder.cantidad--;
                        holder.mCantidad.setText(Integer.toString(holder.cantidad));
                    }
                }
            });
            holder.bCarrito.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if(holder.cantidad != 0) {
                        Log.e("comprando", "guardando item "+ holder.mItem.id);
                        if(CarritoContent.exists(holder.mItem.id)) {
                            CarritoContent.actualizarCantidad(holder.cantidad, holder.mItem.id);
                            Log.e("comprando", "se han añadido al anterior "+ holder.cantidad);
                        }
                        else {

                            CarritoContent.CarritoItem.save(new CarritoContent.CarritoItem(holder.mItem.id,
                                    holder.mItem.nombre, holder.mItem.precio, Integer.toString(holder.cantidad)));
                            Log.e("comprando", "se han añadido nuevo" + holder.cantidad);
                        }
                            holder.cantidad = 0;
                            holder.mCantidad.setText(Integer.toString(holder.cantidad));


                    }
                }
            });


        }

        private void chooseProduct(int currentPos, Context context){

        }



        @Override
        public int getItemCount() {
            return mValues.size();
        }

        public class ViewHolder extends RecyclerView.ViewHolder {
            public final View mView;
            public final TextView mNombreView;
            public final TextView mPrecioView;
            public final ImageView mImagenView;
            public ProductContent.ProductItem mItem;
            public Integer cantidad;
            public final Button bMas;
            public final Button bMenos;
            public final Button bCarrito;
            public final TextView mCantidad;

            public ViewHolder(View view) {
                super(view);
                mView = view;
                mNombreView = (TextView) view.findViewById(R.id.item_nombre);
                mPrecioView = (TextView) view.findViewById(R.id.item_precio);
                mImagenView = (ImageView) view.findViewById(R.id.item_image);
                mCantidad = (TextView) view.findViewById(R.id.cantidad);
                bMas = (Button) view.findViewById(R.id.b_mas);
                bMenos = (Button) view.findViewById(R.id.b_menos);
                bCarrito = (Button) view.findViewById(R.id.b_carrito);
                cantidad = 0;

            }

            @Override
            public String toString() {
                return super.toString() + " '" + mNombreView.getText() + "'";
            }
        }
    }


}
