package edu.uoc.baserrikoa.Control.productor;

import android.app.ActivityOptions;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.Snackbar;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.baserrikoa.Control.MenuManagementActivity;
import edu.uoc.baserrikoa.R;
import edu.uoc.baserrikoa.model.SesionData;
import edu.uoc.baserrikoa.model.VendedorContent;

/**
 * An activity representing a list of Sellers. This activity
 * has different presentations for handset and tablet-size devices. On
 * handsets, the activity presents a list of items, which when touched,
 * lead to a {@link VendedorDetailActivity} representing
 * item details. On tablets, the activity presents the list of items and
 * item details side-by-side using two vertical panes.
 */
public class VendedorListActivity extends MenuManagementActivity {

    private final static String TAG = "VendedorListActivity";
    private SimpleItemRecyclerViewAdapter adapter;
    private DatabaseReference myRef;
    private SwipeRefreshLayout swipeContainer;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_vendedor_list);
        Toolbar toolbar = (Toolbar) findViewById(R.id.my_toolbar);
        setSupportActionBar(toolbar);
        toolbar.setTitle(getTitle());

        swipeContainer = (SwipeRefreshLayout) findViewById(R.id.swipeContainer);
        // Setup refresh listener which triggers new data loading
        swipeContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                refreshVendedorList();
            }
        });
        // Configure the refreshing colors
        swipeContainer.setColorSchemeResources(android.R.color.holo_blue_bright,
                android.R.color.holo_green_light,
                android.R.color.holo_orange_light,
                android.R.color.holo_red_light);

        RecyclerView recyclerView = (RecyclerView) findViewById(R.id.vendedor_list);
        RecyclerView.LayoutManager layoutManager = new GridLayoutManager(VendedorListActivity.this, 2);
        recyclerView.setLayoutManager(layoutManager);
        setupRecyclerView(recyclerView);

        if(SesionData.firebaseConected) downloadVendedores();


    }

    private void downloadVendedores() {
        swipeContainer.setRefreshing(true);

        myRef = SesionData.database.getReference("vendedores");
        // Read from the database
        myRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                getVendedoresFromDataSnapshot(dataSnapshot);

            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.w(TAG, "Failed to read value.", error.toException());
                getVendedoresFromDB();
            }
        });
    }


    private void refreshVendedorList() {
        if (myRef != null) {
            myRef.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    getVendedoresFromDataSnapshot(dataSnapshot);
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {
                    // Failed to read value
                    Log.w(TAG, "Failed to read value.", databaseError.toException());
                    getVendedoresFromDB();
                }
            });
        } else {
            Log.w(TAG, "My Ref null");
            getVendedoresFromDB();
        }
    }

    private void getVendedoresFromDB() {
        List<VendedorContent.VendedorItem> values = VendedorContent.getVendedores();
        adapter.setItems(values);
        swipeContainer.setRefreshing(false);
        Snackbar.make(swipeContainer, "Vendedores updated from DB", Snackbar.LENGTH_LONG).show();
    }

    private void getVendedoresFromDataSnapshot(DataSnapshot dataSnapshot) {
        // This method is called once with the initial value and again
        // whenever data at this location is updated.
        GenericTypeIndicator<ArrayList<VendedorContent.VendedorItem>> genericTypeIndicator =
                new GenericTypeIndicator<ArrayList<VendedorContent.VendedorItem>>() {};
        ArrayList<VendedorContent.VendedorItem> values = dataSnapshot.getValue(genericTypeIndicator);
        // Save data in database
        if(values != null){
        for (VendedorContent.VendedorItem pItem : values) {
            if (!VendedorContent.exists(pItem)) {
                pItem.save();
            }
        }

        adapter.setItems(values);
        swipeContainer.setRefreshing(false);
        Snackbar.make(swipeContainer, "Se ha cargado la lista de vendedores", Snackbar.LENGTH_LONG).show();
        }else{
            swipeContainer.setRefreshing(false);
            Snackbar.make(swipeContainer, "Error al cargar la lista de vendedores", Snackbar.LENGTH_LONG).show();
        }
    }

    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
        adapter = new SimpleItemRecyclerViewAdapter(new ArrayList<VendedorContent.VendedorItem>());
        recyclerView.setAdapter(adapter);
    }


    public class SimpleItemRecyclerViewAdapter
            extends RecyclerView.Adapter<SimpleItemRecyclerViewAdapter.ViewHolder> {

        private List<VendedorContent.VendedorItem> mValues;


        public SimpleItemRecyclerViewAdapter(List<VendedorContent.VendedorItem> items) {
            mValues = items;
        }

        public void setItems(List<VendedorContent.VendedorItem> items) {
            mValues = items;
            notifyDataSetChanged();
        }


        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = null;

            view = LayoutInflater.from(parent.getContext())
                        .inflate(R.layout.vendedor_list_content, parent, false);

            return new ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final ViewHolder holder, final int position) {
            holder.mItem = mValues.get(position);
            holder.mNombreView.setText(mValues.get(position).nombre);
            holder.mLocalidadView.setText(mValues.get(position).localidad);
            if (holder.mImagenView != null) {
                Picasso.with(VendedorListActivity.this).load(mValues.get(position).getImagen())
                            .into(holder.mImagenView);

            }

            holder.mView.setTag(position);
            holder.mView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    int currentPos = (int) v.getTag();
                    Log.d("LIST: ",""+currentPos);
                    Bundle animation = ActivityOptions.makeCustomAnimation(VendedorListActivity.this, R.anim.translate_in_bottom,
                                R.anim.translate_out_bottom).toBundle();
                    Intent intent = new Intent(v.getContext(), VendedorDetailActivity.class);
                    intent.putExtra(VendedorDetailFragment.ARG_ITEM_ID, currentPos);
                    v.getContext().startActivity(intent, animation);


                }
            });
        }

        private void chooseVendedor(int currentPos, Context context){

        }



        @Override
        public int getItemCount() {
            return mValues.size();
        }

        public class ViewHolder extends RecyclerView.ViewHolder {
            public final View mView;
            public final TextView mNombreView;
            public final TextView mLocalidadView;
            public final ImageView mImagenView;
            public VendedorContent.VendedorItem mItem;

            public ViewHolder(View view) {
                super(view);
                mView = view;
                mNombreView = (TextView) view.findViewById(R.id.vendedor_nombre);
                mLocalidadView = (TextView) view.findViewById(R.id.vendedor_localidad);
                mImagenView = (ImageView) view.findViewById(R.id.vendedor_image);
            }

            @Override
            public String toString() {
                return super.toString() + " '" + mNombreView.getText() + "'";
            }
        }
    }


}
