package edu.uoc.baserrikoa.Control.usuario;

import android.annotation.SuppressLint;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.util.ArrayList;

import edu.uoc.baserrikoa.Control.MenuActivity;
import edu.uoc.baserrikoa.Control.MenuManagementActivity;
import edu.uoc.baserrikoa.R;
import edu.uoc.baserrikoa.model.SesionData;
import edu.uoc.baserrikoa.model.UsuarioItem;

/*Actividad que muestra los datos del usuario y permite modificarlos*/
public class UserDetailActivity extends MenuManagementActivity {

    EditText uNombre;
    EditText uApellidos;
    EditText uEmail;
    EditText uDireccion;
    EditText uContrasena;
    EditText uTelefono;
    ImageView uImagen;
    Button bCambiarDatos;

    UsuarioItem uItem;

    private DatabaseReference myRef;

    @SuppressLint("CutPasteId")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_user_detail);

        Toolbar toolbar = (Toolbar) findViewById(R.id.my_toolbar);
        setSupportActionBar(toolbar);
        toolbar.setTitle(R.string.datos_usuario);

        uNombre = (EditText) findViewById(R.id.usuario_nombre);
        uApellidos = (EditText) findViewById(R.id.usuario_apellidos);
        uEmail = (EditText) findViewById(R.id.usuario_email);
        uDireccion = (EditText) findViewById(R.id.usuario_direccion);
        uContrasena = (EditText) findViewById(R.id.usuario_password);
        uTelefono = (EditText) findViewById(R.id.usuario_telefono);
        uImagen = (ImageView) findViewById(R.id.usuario_imagen);
        bCambiarDatos = (Button) findViewById(R.id.bCambiarUsuario);
        bCambiarDatos.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                saveDataInFirebase();
            }
        });
        if(SesionData.firebaseConected) downloadUserData();

    }

    /*Descargar la información relativa al usuario*/
    public void downloadUserData(){

        myRef = SesionData.database.getReference("usuarios").child(SesionData.userID);//Coger solo la información de nuestro usuario

        myRef.addListenerForSingleValueEvent(new ValueEventListener()
        {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    getUserDataFromDataSnapshot(dataSnapshot);
                }

                @Override
                public void onCancelled(DatabaseError error) {
                    // Failed to read value
                    Log.w("usuario: ", "Failed to read value.", error.toException());
                }
            });
    }


    private void getUserDataFromDataSnapshot(DataSnapshot dataSnapshot) {
        // This method is called once with the initial value and again
        // whenever data at this location is updated.

        GenericTypeIndicator<ArrayList<UsuarioItem>> genericTypeIndicator =
                new GenericTypeIndicator<ArrayList<UsuarioItem>>() {};
        ArrayList<UsuarioItem> values = dataSnapshot.getValue(genericTypeIndicator);
        // Save data in database
        if (values != null){
            uItem = values.get(0);
            uItem.save();
            setDataInActivity(uItem);
        }

    }

    /*Guardar datos del usuario, ya sea por primera vez o hayan sido modificados*/
    private void saveDataInFirebase()
    {
        myRef.child("0").child("nombre").setValue(uNombre.getText().toString());
        myRef.child("0").child("apellidos").setValue(uApellidos.getText().toString());
        myRef.child("0").child("telefono").setValue(uTelefono.getText().toString());
        myRef.child("0").child("email").setValue(uEmail.getText().toString());
        myRef.child("0").child("contrasena").setValue(uContrasena.getText().toString());
        myRef.child("0").child("direccion").setValue(uDireccion.getText().toString());

        //Mostrar alerta de datos guardados
        AlertDialog.Builder builder = new AlertDialog.Builder(UserDetailActivity.this);
        builder.setMessage("Datos actualizados")
                .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        Intent intent = new Intent(UserDetailActivity.this, MenuActivity.class);
                        startActivity(intent);
                    }
                });
        builder.create();
        builder.show();

    }

    /*Mostrar la información en pantalla*/
    private void setDataInActivity(UsuarioItem uItem){
        if(uItem != null) {
            if (uItem.nombre != null)
                uNombre.setText(uItem.nombre);

            if (uItem.telefono != null)
                uTelefono.setText(uItem.telefono);


            if (uItem.apellidos != null)
                uApellidos.setText(uItem.apellidos);

            if (uItem.email != null)
                uEmail.setText(uItem.email);

            if (uItem.direccion != null)
                uDireccion.setText(uItem.direccion);

            if (uItem.contrasena != null)
                uContrasena.setText(uItem.contrasena);

            if (uItem.imagen != null)
                Picasso.with(this).load(uItem.imagen).into(uImagen);
        }
    }


}
